<?php
/**
 * Class WPAUST_Process
 *
 * Allow each author to share their custom status.
 * The status can be shown in anywhere of their posts.
 *
 * @package WP Author Status v2.0 - 30 January, 2025
 * @link https://www.realwebcare.com/
 */
if ( ! defined('ABSPATH') ) {
    exit;
}

if ( ! class_exists('WPAUST_Process') ) {
    class WPAUST_Process {

        private static $instance;

        private $user_info;
        private $user_id;

        private function __construct() {
            // Load necessary required files.
            $this->required_files();

            // Adding dashboard widgets
			add_action('wp_dashboard_setup', array( $this, 'render_author_status_widgets' ) );

            // Fetch Dashboard Widget Update
            add_action( 'wp_ajax_wpaust-dashboard-form-action', array( $this, 'fetch_wp_author_status' ) );

            // Update user status
            add_action( 'wp_ajax_wpaust-user-status-action', array( $this, 'fetch_wp_author_status' ) );

            // filter the content
            add_filter( 'the_content', array( $this, 'wpaust_add_shortcode_before_content' ) );
        }

        public static function get_instances() {
            if ( self::$instance ) {
                return self::$instance;
            }

            self::$instance = new self();

            return self::$instance;
        }

        /**
         * Includes required files for the plugin's functionality.
         *
         * This method loads the necessary PHP files that provide core functionality for the plugin. 
         * By requiring these files, the plugin ensures that all actions, options, and shortcodes 
         * are properly initialized and available.
         *
         * @return void
         */
        private function required_files() {
            //Handles the enqueueing of scripts and styles.
            require_once WPAUST_PLUGIN_PATH . 'includes/wpaust-enqueue.php';

            // Manages shortcodes for user interactions.
            require_once WPAUST_PLUGIN_PATH . 'settings/wpaust-options.php';

            // Manages shortcodes for user interactions.
            require_once WPAUST_PLUGIN_PATH . 'includes/wpaust-shortcode.php';

            // Initializes any additional functions required by the plugin.
            require_once WPAUST_PLUGIN_PATH . 'settings/wpaust-functions.php';
        }

		function render_author_status_widgets() {
			if ( ! current_user_can( 'delete_published_posts' ) )  {
				wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'wp-author-status' ) );
			}

            // Check if the platform is admin or not
            if ( ! is_admin() ) {
                return;
            }

			// Adds a new dashboard widget.
			wp_add_dashboard_widget( 'author_status_widget', 'Author Status Widget', array( $this, 'render_dashboard_author_status' ) );
		}

        /**
         * Create or update a default author status for a user.
         *
         * @param int $user_id The ID of the user.
         * @return array The author status array.
         */
        public function handle_wp_author_status( $user_id ) {
            // Validate the user ID
            if ( ! is_int( $user_id ) || $user_id <= 0 ) {
                return wp_send_json_error(array(
                    'message' => esc_html__( 'User not idetified!', 'wp-author-status' )
                ));
            }

            // Fetch user data
            $user_info = get_userdata( $user_id );
            if ( ! $user_info ) {
                return esc_html__( 'User does not exist!', 'wp-author-status' );
            }
            
	        $old_status = get_user_meta( $user_id, 'author_status', true );
	        $old_status_date = get_user_meta( $user_id, 'status_date', true );

            // Define meta keys
            $status_key = 'wpaust_author_status';
            $date_key = 'wpaust_status_date';

            if ( ! empty( $old_status ) ) {
                update_user_meta( $user_id, $status_key, sanitize_text_field( $old_status ) );
                delete_user_meta( $user_id, 'author_status' );
            }

            if ( ! empty( $old_status_date ) ) {
                update_user_meta( $user_id, $date_key, sanitize_text_field( $old_status_date ) );
                delete_user_meta( $user_id, 'status_date' );
            }

            // Get current status and date meta
            $author_status = get_user_meta( $user_id, $status_key, true );
            $status_date = get_user_meta( $user_id, $date_key, true );

            // Get User Name
            $user_name = isset( $user_info->user_nicename ) ? ucfirst( $user_info->display_name ) : ucfirst($user_info->user_login);

            // Generate a default status if not present
            if ( empty( $author_status ) ) {
                $author_status = sprintf(
                    'Dear Authority, I am %s. It is an honor to be part of your esteemed network, contributing meaningful and high-quality articles for your audience.',
                    esc_html( ucfirst( $user_name ) )
                );

                // Sanitize and save the default status
                update_user_meta( $user_id, $status_key, sanitize_text_field( $author_status ) );
            }

            // Generate a default date if not present
            if ( empty( $status_date ) ) {
                $status_date = current_time( 'mysql' );

                // Save the default date
                update_user_meta( $user_id, $date_key, sanitize_text_field( $status_date ) );
            }

            // Default configuration for the status
            $defaults = array(
                'wpaust_author_status' => sanitize_text_field( $author_status ),
                'wpaust_status_date'   => sanitize_text_field( $status_date ),
            );

            // Ensure author status is an array
            if ( ! is_array( $author_status ) ) {
                $author_status = array();
            }

            // Merge defaults with the existing status
            $author_status = array_merge( $defaults, $author_status );

            return $author_status;
        }

        /**
         * Fetches and updates the author status.
         * 
         * This method verifies the nonce, checks the user ID, and ensures the user exists.
         * It then processes the author status update if valid data is submitted.
         * Depending on user capabilities, it updates the user meta and returns the status update success or failure response.
         */
        public function fetch_wp_author_status() {
            // Nonce verification
            check_ajax_referer( 'wpaust_update_status', '_wpaust_nonce' );

            // Ensure the user ID is valid
            $user_id = isset( $_POST['user_id'] ) ? intval( $_POST['user_id'] ) : 0;

            if ( ! is_int( $user_id ) || $user_id <= 0 ) {
                return wp_send_json_error(array(
                    'message' => wp_kses_post( '<h4>Error!</h4>User not idetified!', 'wp-author-status' )
                ) );
            }

            // Check if the user exists
            $user = get_user_by( 'ID', $user_id );
            if ( ! $user ) {
                return wp_send_json_error(array(
                    'message' => wp_kses_post( '<h4>Error!</h4>User does not exist.', 'wp-author-status' )
                ), 404);
            }

            // Handle the author status update (if any)
            $status = isset( $_POST['author_status'] ) ? wp_unslash( sanitize_textarea_field( $_POST['author_status'] ) ) : '';
            if ( empty( $status ) ) {
                return wp_send_json_error(array(
                    'message' => wp_kses_post( '<h4>Error!</h4>Status cannot be empty.', 'wp-author-status' )
                ) );
            }

            if ( empty( $_POST['author_status_submit'] ) ) {
                return wp_send_json_error(array(
                    'message' => wp_kses_post( '<h4>Error!</h4>Form submission data missing!', 'wp-author-status' )
                ) );
            }

            // Handle form submission
            if ( $_POST['author_status_submit'] === 'yes' ) {
                if ( current_user_can( 'delete_published_posts' ) ) {
                    // Allow full content for users with delete_published_posts capability
                    $author_status['wpaust_author_status'] = wp_unslash( sanitize_textarea_field( $_POST['author_status'] ) );
                } else {
                    // Restrict content for other users
                    $author_status['wpaust_author_status'] = wp_unslash( wp_filter_post_kses( sanitize_textarea_field( $_POST['author_status'] ) ) );

                    wp_send_json_error( array(
                        'message' => wp_kses_post( '<h4>Error!</h4>You do not have permission to update status!', 'wp-author-status' )
                    ) );
                }

                // Update the user meta
                $status = $author_status['wpaust_author_status'];
                $status_date = current_time( 'mysql' ); // Update date to the current time

                update_user_meta( $user_id, 'wpaust_author_status', $status );
                update_user_meta( $user_id, 'wpaust_status_date', $status_date );

                // Assuming `$_POST['status_date']` is the new status time from the form
                $status_date_timestamp = ! empty( $status_date ) ? strtotime( $status_date ) : '';
                $current_timestamp = current_time( 'timestamp' );

                // The relative time since the status was last updated
                $time_diff = human_time_diff($status_date_timestamp, $current_timestamp);

                wp_send_json_success(array(
                    'timeago' => $time_diff,
                    'message' => wp_kses_post('<h4>Success!</h4>Your Status Updated Successfully!', 'wp-author-status')
                ) );
            } else {
                wp_send_json_error( array(
                    'message' => wp_kses_post( '<h4>Error!</h4>Something is wrong!', 'wp-author-status' )
                ) );
            }
        }

        /**
         * Renders the author status dashboard.
         *
         * This method checks if the current user has the required capabilities to update the status. 
         * It fetches the current user ID, validates it, and ensures the user exists.
         * If the user is valid, it displays the current status and allows the user to update it via a form.
         * The form includes nonce verification and updates the user meta when submitted. 
         * Additionally, it calculates and displays the time difference since the last update.
         */
        public function render_dashboard_author_status() {
            if ( ! current_user_can( 'delete_published_posts' ) ) {
                return;
            }
        
            // Get the current user ID
            $user_id = get_current_user_id();
        
            // Ensure the user ID is valid
            if ( ! is_int( $user_id ) || $user_id <= 0 ) {
                return esc_html__( 'Invalid user ID.', 'wp-author-status' );
            }
        
            // Check if the user exists
            $user = get_user_by( 'ID', $user_id );
            if ( ! $user ) {
                return esc_html__( 'User does not exist.', 'wp-author-status' );
            }
        
            // Get the status
            $author_status = $this->handle_wp_author_status( $user_id );
            $status_date   = get_user_meta( $user_id, 'wpaust_status_date', true );
            $statusdate    = strtotime( $status_date );
            $current_date  = current_time( 'timestamp' );
        
            // Check and update the status on form submission
            if ( isset( $_POST['author_status_submit'] ) && isset( $_POST['_wpnonce'] ) && wp_verify_nonce( $_POST['_wpnonce'], 'wpaust_update_status' ) ) {  
                if ( current_user_can( 'delete_published_posts' ) ) {
                    $author_status['wpaust_author_status'] = sanitize_textarea_field( $_POST['author_status'] );
                } else {
                    $author_status['wpaust_author_status'] = sanitize_textarea_field( wp_filter_post_kses( $_POST['author_status'] ) );
                }
        
                // Update user meta
                update_user_meta( $user_id, 'wpaust_author_status', $author_status['wpaust_author_status'] );
                update_user_meta( $user_id, 'wpaust_status_date', current_time( 'mysql' ) );
            }
        
            // Allowed HTML tags for wp_kses()
            $allowed_tags = array(
                'div'      => array( 'id' => array(), 'class' => array() ),
                'h3'       => array(),
                'h4'       => array(),
                'p'        => array(),
                'form'     => array( 'id' => array(), 'method' => array(), 'action' => array() ),
                'textarea' => array( 'id' => array(), 'name' => array(), 'class' => array() ),
                'input'    => array( 'type' => array(), 'value' => array(), 'class' => array(), 'name' => array() ),
                'button'   => array( 'type' => array(), 'id' => array(), 'class' => array() ),
                'span'     => array( 'id' => array(), 'class' => array() ),
            );
        
            // Generate form HTML
            ob_start(); ?>
            <div class="wpaust-author-status-widget">
                <div class="wpaust-status-display">
                    <h3>
                        <?php
                        printf(
                            __( '%s Status', '' ),
                            esc_html( $user->display_name ),
                        );
                        ?>
                        <span id="wpaust-loader" class="wpaust-hidden"></span>
                    </h3>
                    <div id="wpaust-status-body">
                        <form id="wpaust-author-dashboard-form" method="post" action="">
                            <?php wp_nonce_field( 'wpaust_update_status' ); ?>
        
                            <textarea id="wpaust-author-status" name="author_status"><?php echo esc_textarea( $author_status['wpaust_author_status'] ); ?></textarea>
        
                            <input type="hidden" name="action" value="wpaust-dashboard-form-action">
                            <input type="hidden" name="user_id" value="<?php echo absint( $user_id ) ?>">
                            <input type="hidden" name="author_status_submit" value="yes">
                            <button id="wpaust-dashborad-button" class="button-primary" type="submit"><?php esc_html_e( 'Update Status', 'wp-author-status' ); ?></button>
                        </form>

                        <div class="wpaust-status-date-show">
                            <?php echo esc_html( human_time_diff( $statusdate, $current_date ) . ' ago' ); ?>
                        </div>
                    </div>
                    <div id="wpaust-modal-message" class="wpaust-status-modal wpaust-hidden">
                        <div class="wpaust-modal-content wpaust-success-message">
                            <p></p>
                        </div>
                    </div>
                </div>
            </div>
            <?php
            echo wp_kses( ob_get_clean(), $allowed_tags );
        }

        /**
         * Adds the author status shortcode before or after the content on single post pages.
         *
         * This method retrieves the general section settings for auto-inserting the user status. 
         * If auto-insertion is enabled, it checks if the current page is a single post and ensures it's the main query.
         * It then generates the user status using the shortcode and inserts it either before or after the content 
         * based on the specified position setting.
         */
        public function wpaust_add_shortcode_before_content( $content ) {
            // Retrieving General Section Data
            $wpaust_autosc = wpaust_get_option( 'wpaust_autosc', 'wpaust_generals', 'on' );
            $wpaust_scposn = wpaust_get_option( 'wpaust_scposn', 'wpaust_generals', 'before' );

            // Return if Auto Insert User Status is Off
            if( ! isset( $wpaust_autosc ) || $wpaust_autosc != 'on' ) {
                return $content;
            }

            if ( is_single() && in_the_loop() && is_main_query() && ! is_page() && get_post_type() == 'post' ) {
                $status_output = do_shortcode('[wp-author-status]');

                // Check where to display the user status
                if ( $wpaust_scposn == 'before' ) {
                    $content = $status_output . $content;
                } else {
                    $content = $content . $status_output;
                }
            }
            return $content;
        }
    }
}

WPAUST_Process::get_instances();
