<?php
/**
 * Class WPAUST_Shortcode
 *
 * Allow each author to share their custom status.
 * The status can be shown in anywhere of their posts.
 *
 * @package WP Author Status v2.0 - 30 January, 2025
 * @link https://www.realwebcare.com/
 */
if ( ! defined('ABSPATH') ) {
    exit;
}

if ( ! class_exists('WPAUST_Shortcode') ) {
    class WPAUST_Shortcode {

        private static $instance;

        private function __construct() {
            // Add shortcode
            add_shortcode( 'wp-author-status', array( $this, 'user_status_shortcode' ) );
        }

        public static function get_instances() {
            if ( self::$instance ) {
                return self::$instance;
            }

            self::$instance = new self();

            return self::$instance;
        }

        public function user_status_shortcode( $atts ) {
            global $post;
            // Sanitize and validate shortcode attributes
            $atts = shortcode_atts(
                array(
                    'status' => '',
                    'colors' => 'bp',
                ),
                $atts,
                'wp-author-status'
            );

            $user_id = $post->post_author;
            $user_info = get_user_by( 'ID', $user_id );

            // Validate the user ID
            $user_id = isset( $user_id ) && is_numeric( $user_id ) ? intval( $user_id ) : 0;
        
            // Get and sanitize user metadata
            $status = sanitize_textarea_field( get_user_meta( $user_id, 'wpaust_author_status', true ) );
            $status_date = sanitize_text_field( get_user_meta( $user_id, 'wpaust_status_date', true ) );

            // Validate and escape the author status
            $author_status = ! empty( $status ) ? wp_kses_post( $status ) : esc_html__( 'No status available.', 'wp-author-status' );
        
            // Convert date to timestamp and validate
            $status_date_timestamp = $status_date ? strtotime( $status_date ) : false;
            $current_timestamp = current_time( 'timestamp' );
        
            // Use shortcode attribute for status if provided and valid
            if ( ! empty( $atts['status'] ) ) {
                $author_status = sanitize_text_field( $atts['status'] );
            }

            // User choiced color
            $wpaust_colors = wpaust_get_option( 'wpaust_colors', 'wpaust_generals', 'bp' );

            // display or not
            $wpaust_authnm = wpaust_get_option( 'wpaust_authnm', 'wpaust_usstatus', 'on' );
            $wpaust_autimg = wpaust_get_option( 'wpaust_autimg', 'wpaust_usstatus', 'on' );
            $wpaust_pstcnt = wpaust_get_option( 'wpaust_pstcnt', 'wpaust_usstatus', 'on' );
            $wpaust_howago = wpaust_get_option( 'wpaust_howago', 'wpaust_usstatus', 'on' );

            // Shortcode attribute for colors if provided and valid
            $status_color = ! empty( $atts['colors'] ) && $atts['colors'] != 'bp' ? sanitize_text_field( $atts['colors'] ) : sanitize_text_field ( $wpaust_colors );

            // Safely retrieve and format user details
            $user_name = isset( $user_info->user_nicename )
                ? esc_html( ucfirst( $user_info->display_name ) )
                : esc_html( ucfirst( $user_info->user_login ) );
        
            // Get total posts by the user
            $total_posts = get_posts(
                array(
                    'author'         => $user_id,
                    'post_status'    => 'publish',
                    'post_type'      => 'post',
                    'posts_per_page' => -1,
                )
            );
            $total_posts_count = count( $total_posts );
        
            // Determine singular or plural for post count
            $singular_post = $total_posts_count === 1 ? '' : 's';
        
            ob_start();
            ?>
            <div id="wpaust-status-wrap" class="wpaust-<?php echo esc_attr( $status_color ); ?>-gradient">
                <div id="wpaust-status-header">
                    <?php if ( isset( $wpaust_autimg ) && $wpaust_autimg == 'on' ) { ?>
                        <div class="wpaust-user-avatar">
                            <?php echo get_avatar( $user_id, '', 'mystery', esc_attr( $user_name ) ); ?>
                        </div>
                    <?php } ?>

                    <?php if ( isset( $wpaust_authnm ) && $wpaust_authnm == 'on' ) { ?>
                        <div class="wpaust-user-name"><?php echo esc_html( $user_name ); ?></div>
                    <?php } ?>

                    <?php if ( isset( $wpaust_pstcnt ) && $wpaust_pstcnt == 'on' ) { ?>
                        <div class="wpaust-total-post-count">
                            <?php
                            echo sprintf(
                                /* translators: 1: Total number of posts, 2: Singular form of the word "Post" */
                                esc_html__( '%1$s Post%2$s', 'wp-author-status' ),
                                absint( $total_posts_count ),
                                esc_html( $singular_post )
                            );
                            ?>
                        </div>
                    <?php } ?>
                </div>

                <div class="wpaust-status-info">
                    <div class="wpaust-status-text">
                        <p><?php echo esc_html( $author_status ); ?></p>
                    </div>
                    <?php if ( empty( $atts['status'] ) && isset( $wpaust_howago ) && $wpaust_howago == 'on' ) { ?>
                    <p class="wpaust-status-time">
                        <?php
                        echo esc_html( human_time_diff( $status_date_timestamp, $current_timestamp ) );
                        esc_html_e( ' ago', 'wp-author-status' );
                        ?>
                    </p>
                    <?php } ?>
                </div>
            </div>
            <?php
            return ob_get_clean();
        }
    }
}

WPAUST_Shortcode::get_instances();
