<?php
/**
 * Handles the display of the plugin sidebar content.
 *
 * The WPAUST_Sidebar class is responsible for rendering the plugin's shortcode usage 
 * instructions and additional plugin information in the admin dashboard sidebar. 
 * It provides flexibility to control the visibility of notes, plugin info, 
 * and allows for custom CSS class inclusion for styling.
 * 
 * @package WP Author Status v2.0 - 30 January, 2025
 * @link https://www.realwebcare.com/
 */
if ( ! defined( 'ABSPATH' ) ) exit;

if ( !class_exists('WPAUST_Sidebar' ) ) {
	class WPAUST_Sidebar {
        private static $instance;

        private function __construct() {

        }

        public static function get_instances() {
            if ( self::$instance ) {
                return self::$instance;
            }

            self::$instance = new self();

            return self::$instance;
        }

        /**
         * Renders the plugin sidebar with shortcode usage and plugin information.
         *
         * @param bool   $show_note Determines whether to display the shortcode usage instructions. Default is true.
         * @param bool   $show_info Determines whether to display the plugin information. Default is true.
         * @param string $class     Optional. Custom CSS class for the sidebar container. Default is an empty string.
         *
         * @return void
         */
        public function wpaust_sidebar( $show_note = true, $show_info = true, $class = '' ) {
            if ( ! empty( $class ) ) {
                $class = ' ' . $class;
            }
            $content = '';
            $content .= '
            <div id="wpaust-sidebar" class="postbox-container'.esc_attr( $class ).'">';
                // Instruction
                if ( $show_note ) {
                    $content .= '
                    <div id="wpaustusage-note" class="wpaustusage-sidebar">
                        <h3>'.esc_html__('Code Usage Instruction', 'wp-author-status').'</h3>
                        <div class="wpaust">
                            <p class="wpaust-first">To display the author status shortcode in a WordPress post or page, you need to access the post or page editor in the WordPress dashboard. Here\'s how:</p>
                            <ol>
                                <li>Go to Posts menu to display the author status.</li>
                                <li>Either create a new post, or edit an existing one.</li>
                                <li>Locate the spot in the post where you want to display the status.</li>
                                <li>Paste the following shortcode into the editor:
                                    <pre><code>[wp-author-status]</code></pre>
                                </li>
                                <li>publish or update the post.</li>
                            </ol>
                            <p>Once you\'ve published or updated the post, the shortcode will be processed and the author status will be displayed on the front-end post of your site.</p>
                        </div>
                    </div>';
                }
        
                // Plugin Info
                if ( $show_info ) {
                    $content .= '
                    <div id="wpaustusage-info" class="wpaustusage-sidebar">'.
                        /* translators: %1$s: URL for Realwebcare website, %2$s: URL for Realwebcare Facebook page, %3$s: URL for plugin support page, %4$s: URL for plugin review page */
                        sprintf(__('
                            <h3>Plugin Info</h3>
                            <ul class="wpaustusage-list">
                                <li>WP Author Status</li>
                                <li>Version: 2.0</li>
                                <li>Scripts: PHP + CSS + JS</li>
                                <li>Requires: Wordpress 5.2</li>
                                <li>First Released: 16 June, 2014</li>
                                <li>Last Updated: 30 January, 2025</li>
                                <li>By: <a href="%1$s" target="_blank">Realwebcare</a></li>
                                <li>Facebook Page: <a href="%2$s" target="_blank">Realwebcare</a></li>
                                <li>Need Help? <a href="%3$s" target="_blank">Support</a></li>
                                <li>Like it? Please leave us a <a target="_blank" href="%4$s">&#9733;&#9733;&#9733;&#9733;&#9733;</a> rating. We highly appreciate your support!</li>
                            </ul>', 'wp-author-status' ),
                            esc_url( "https://www.realwebcare.com/" ),
                            esc_url( "https://www.facebook.com/realwebcare" ),
                            esc_url( "https://wordpress.org/support/plugin/wp-author-status/" ),
                            esc_url( "https://wordpress.org/support/plugin/wp-author-status/reviews/?filter=5/#new-post" )
                        ).'
                    </div>';
                }
            $content .= '</div>';
        
            return wp_kses_post($content);
        }
    }
}
