<?php
/**
 * Class WPAUST_Functions
 *
 * This file contains functions that play a crucial role in the initial setup
 * of the "WP Author Status" plugin. These functions handle tasks such as
 * text domain setup for translations, adding action links to the plugin settings,
 * and various other essential tasks needed when the plugin is live at the front-end.
 *
 * @package WP Author Status v2.0 - 30 January, 2025
 * @link https://www.realwebcare.com/
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if(!class_exists('WPAUST_Functions')) {
    class WPAUST_Functions {

        private static $instance;

        public function __construct() {
            /// Load plugin text domain
            add_action( 'plugins_loaded', array( $this, 'wpaust_textdomain' ) );

            // Init admin notice for review
            add_action( 'admin_init', array( $this, 'wpaust_check_i10n_date' ) );

            // Init to remove admin notice
            add_action( 'admin_init', array( $this, 'wpaust_spare_me' ), 5 );

            // This filter allows us to modify the action links displayed on the Plugins page
            add_filter( 'plugin_action_links_' . plugin_basename(WPAUST_AUF), array( $this, 'wpaust_plugin_actions' ) );
        }

        public static function get_instances() {
            if ( self::$instance ) {
                return self::$instance;
            }

            self::$instance = new self();

            return self::$instance;
        }

        /**
         * Load the plugin text domain for translation.
         *
         * This function loads the `.mo` files for translating the plugin
         * into the user's current locale.
         *
         * @return void
         */
        public function wpaust_textdomain() {
            $locale = apply_filters( 'plugin_locale', get_locale(), 'wp-author-status' );
            load_textdomain( 'wp-author-status', trailingslashit( WP_PLUGIN_DIR ) . 'wp-author-status/languages/wp-author-status-' . $locale . '.mo' );
            load_plugin_textdomain( 'wp-author-status', false, plugin_dir_path( __FILE__ ) . 'languages' );
        }

        /* Add plugin action links */
        public function wpaust_plugin_actions( $links ) {
            $settings_url = esc_url( menu_page_url( 'wpaust-options', false ) );
            $settings_url = wp_nonce_url( $settings_url, 'wpaust_settings_action' );

            $support_url = esc_url( "https://wordpress.org/support/plugin/wp-author-status" );

            $links[] = '<a href="'. $settings_url .'">'. esc_html__('Settings', 'wp-author-status') .'</a>';
            $links[] = '<a href="'. $support_url .'" target="_blank">'. esc_html__('Support', 'wp-author-status') .'</a>';
            return $links;
        }

        /**
        * Check date on admin initiation and add to admin notice if it was over 7 days ago.
        * @return null
        */
        public function wpaust_check_i10n_date() {
            // Retrieve the 'spare_me' option
            $spare_me = get_option( 'wpaust_spare_me', false );
        
            // Proceed only if 'spare_me' is not set
            if ( !$spare_me ) {
                // Retrieve the 'i10n_date' option and set a default value if not found
                $i10n_date = get_option( 'wpaust_a08n_time', false );
        
                // Calculate the past date for the threshold (7 days in seconds: 7 * 24 * 60 * 60)
                $past_date = strtotime( '-7 days' );
        
                // Validate the 'i10n_date' and compare it with the threshold
                if ( $i10n_date && is_numeric( $i10n_date ) && $i10n_date < $past_date ) {
                    // If the condition is met, display the admin notice
                    add_action( 'admin_notices', array( $this, 'wpaust_display_admin_notice' ) );
                } else {
                    // Otherwise, store the current timestamp as the activation time
                    $current_time = time(); // Current timestamp
                    update_option( 'wpaust_a08n_time', $current_time );
                }
            }
        }
        
        /**
        * Display Admin Notice, asking for a review
        **/
        public function wpaust_display_admin_notice() {
            // WordPress global variable
            global $pagenow;
        
            // Validate current admin page and query parameters
            if ( is_admin() && $pagenow === 'admin.php' && isset( $_GET['page'] ) && sanitize_text_field( wp_unslash( $_GET['page'] ) ) === 'wpaust-options' ) {
                
                // Generate URLs with proper escaping
                $dont_disturb = esc_url_raw( admin_url( 'admin.php?page=wpaust-options&spare_me=1' ) );
                $dont_disturb = wp_nonce_url( $dont_disturb, 'wpaust_disturb_action' );
                
                // Retrieve plugin data securely
                $plugin_info = get_plugin_data( WPAUST_AUF, true, true );
                
                // Validate and sanitize plugin data
                $text_domain = !empty( $plugin_info['TextDomain'] ) ? sanitize_title( $plugin_info['TextDomain'] ) : 'wp-author-status';
        
                // Construct the review URL securely
                $review_url = 'https://wordpress.org/support/plugin/' . $text_domain . '/reviews/';
                $review_url = wp_nonce_url( $review_url, 'wpaust_review_action' );
        
                // Output the notice with proper escaping
                printf(
                    '<div id="wpaust-review" class="wpaust-notice wpaust-notice-success wpaust-is-dismissible">
                        <span class="wpaust-close-icon">&times;</span>
                        <p>%1$s</p>
                        <p>%2$s</p>
                        <div class="wpaust-review-btn">
                            <a href="%3$s" class="button button-primary" target="_blank">%4$s</a>
                            <a href="%5$s" class="wpaust-grid-review-done button button-secondary">%6$s</a>
                        </div>
                    </div>',
                    esc_html__( 'It\'s been 7 days since your last update or installation of the latest version of ', 'wp-author-status' ) . '<b>WP Author Status</b>' . esc_html__( '! We hope you\'ve had a positive experience so far.', 'wp-author-status' ),
                    esc_html__( 'Your feedback is important to us and can help us improve. If you find our ', 'wp-author-status' ) . '<b>WP Author Status</b>' . esc_html__( ' plugin valuable, could you please take a moment to share your thoughts by leaving a quick review?', 'wp-author-status' ),
                    esc_url( $review_url ),
                    esc_html__( 'Leave a Review', 'wp-author-status' ),
                    esc_url( $dont_disturb ),
                    esc_html__( 'Already Left a Review', 'wp-author-status' )
                );
            }
        }

        /**
        * remove the notice for the user if review already done or if the user does not want to
        **/
        public function wpaust_spare_me() {    
            // Check if 'spare_me' parameter is set and not empty
            if ( isset( $_GET['spare_me'] ) && !empty( $_GET['spare_me'] ) ) {
                // Sanitize the input value to ensure it is safe to use
                $spare_me = sanitize_text_field( wp_unslash( $_GET['spare_me'] ) );
        
                // Validate the value to check if it is the expected value
                if ( $spare_me === '1' ) {
                    // Add the 'wpaust_spare_me' option with a boolean value
                    add_option( 'wpaust_spare_me', true );
                }
            }
        }
    }
}

WPAUST_Functions::get_instances();