<?php
/**
 * Class WPAUST_Settings_Config
 *
 * This class handles the plugin's settings configuration in the WordPress admin dashboard. 
 * It utilizes the Settings API to create and manage settings sections, fields, and pages.
 * It also includes functionality for displaying a help page, adding menu items, and retrieving pages.
 * 
 * @package WP Author Status v2.0 - 30 January, 2025
 * @link https://www.realwebcare.com/
 */
if ( ! defined( 'ABSPATH' ) ) exit;

if ( !class_exists('WPAUST_Settings_Config' ) ) :
	class WPAUST_Settings_Config {

		private $settings_api;
		private $sidebar_info;

		public function __construct() {
			// Load necessary required files.
            $this->required_files();

			// Access the Settings API
			$this->settings_api = new WPAUST_WeDevs_Settings_API;

			// Access the Sidebar
            $this->sidebar_info = WPAUST_Sidebar::get_instances();

			// Sets up the settings sections and fields
			add_action( 'admin_init', array($this, 'admin_init') );

			// Plugin's menu and submenu
			add_action( 'admin_menu', array($this, 'admin_menu') );
		}

		/**
		 * Includes necessary files for the settings API and sidebar classes.
		 *
		 * This method ensures that the required dependencies for the settings API and 
		 * sidebar functionalities are properly loaded.
		 *
		 * @return void
		 */
        private function required_files() {
            require_once WPAUST_PLUGIN_PATH . 'settings/wpaust-class.settings-api.php';
            require_once WPAUST_PLUGIN_PATH . 'includes/wpaust-sidebar.php';
        }

		/**
		 * Initializes the plugin's settings sections and fields.
		 *
		 * This method sets up the settings sections and fields using the settings API
		 * and registers them for use in the admin panel.
		 *
		 * @return void
		 */
		public function admin_init() {

            //set the settings
            $this->settings_api->set_sections( $this->get_settings_sections() );
            $this->settings_api->set_fields( $this->get_settings_fields() );
    
            //initialize settings
            $this->settings_api->admin_init();
		}

		/**
		 * Creates the plugin's menu and submenu pages in the WordPress dashboard.
		 *
		 * This method uses WordPress functions `add_menu_page` and `add_submenu_page` 
		 * to add the plugin options menu and help menu.
		 *
		 * @return void
		 */
		public function admin_menu() {
            add_menu_page(
                esc_html__( 'WP Author Status', 'wp-author-status' ),
                esc_html__( 'Author Status', 'wp-author-status' ),
                'administrator',
                'wpaust-options',
                array( $this, 'wpaust_options_page' ),
                'dashicons-id',
                66
            );

            // Options
            add_submenu_page(
                'wpaust-options',
                esc_html__( 'WP Author Status Options', 'wp-author-status' ),
                esc_html__( 'WP Author Status', 'wp-author-status' ),
                'administrator',
                'wpaust-options',
                array( $this, 'wpaust_options_page' )
            );

			// Help
            add_submenu_page(
                'wpaust-options',
                esc_html__( 'WP Author Status Guide', 'wp-author-status' ),
                esc_html__( 'Help', 'wp-author-status' ),
                'administrator',
                'wpaust-help',
                array( $this, 'wpaust_help_page' )
            );
		}

		/**
		 * Retrieves all the settings sections for the plugin.
		 *
		 * This method defines and returns the sections used in the plugin's settings page.
		 * Each section contains a unique ID and a title, which are displayed in the settings interface.
		 *
		 * @return array Array of settings sections with their IDs and titles.
		 */
		public function get_settings_sections() {
			$sections = array(
				array(
                    'id'    => 'wpaust_generals',
                    'title' => __( 'General Settings', 'wp-author-status' )
                ),
				array(
                    'id'    => 'wpaust_usstatus',
                    'title' => __( 'Status Settings', 'wp-author-status' )
                ),
			);
			return $sections;
		}

		/**
		 * Retrieves all the settings fields from the settings API class.
		 *
		 * This method provides the list of settings fields configured in the plugin's settings API.
		 *
		 * @return array Array of settings fields.
		 */
		public function get_settings_fields() {
			$settings_fields = array(
				'wpaust_generals' => array(
					array(
                        'name'              => 'wpaust_autosc',
                        'label'             => __( 'Auto Insert Author Status:', 'wp-author-status' ),
                        'desc'              => __( 'Automatically add the user status before the content of all single posts.', 'wp-author-status' ),
                        'type'              => 'checkbox',
                        'default'           => 'on',
                        'sanitize_callback' => 'sanitize_text_field'
                    ),
					array(
                        'name'              => 'wpaust_scposn',
                        'label'             => __( 'Author Status Position:', 'wp-author-status' ),
                        'desc'              => __( 'Choose where to display the user status in single posts.', 'wp-author-status' ),
                        'type'              => 'radio',
                        'options'           => array(
                            'before'        => 'Before Content',
                            'after'         => 'After Content'
                        ),
                        'sanitize_callback' => 'sanitize_text_field'
                    ),
					array(
                        'name'              => 'wpaust_colors',
                        'label'             => __( 'Author Status Color:', 'wp-author-status' ),
                        'desc'              => __( 'Select the color combination for the author status display.', 'wp-author-status' ),
                        'type'              => 'select',
                        'options'           => array(
                            'bp'            => 'Blue, Purple',
                            'tb'            => 'Teal, Blue',
                            'gy'            => 'Green, Yellow',
                            'dp'            => 'Dark Purple, Light Purple',
                            'db'            => 'Dark Blue, Light Blue',
                        ),
                        'sanitize_callback' => 'sanitize_text_field'
                    ),
				),
				'wpaust_usstatus' => array(
					array(
						'name'				=> 'wpaust_authnm',
						'label'				=> __( 'Display Author Name:', 'wp-author-status' ),
						'desc'				=> __( 'Mark to show the author name in the author status.', 'wp-author-status' ),
						'type'				=> 'checkbox',
						'default'			=> 'on',
						'sanitize_callback' => 'sanitize_text_field'
					),
					array(
						'name'				=> 'wpaust_autimg',
						'label'				=> __( 'Display Author Gravatar:', 'wp-author-status' ),
						'desc'				=> __( 'Mark this option to display the author\'s Gravatar next to their status.', 'wp-author-status' ),
						'type'				=> 'checkbox',
						'default'			=> 'on',
						'sanitize_callback' => 'sanitize_text_field'
					),
					array(
						'name'				=> 'wpaust_pstcnt',
						'label'				=> __( 'Display Author Post Count:', 'wp-author-status' ),
						'desc'				=> __( 'Mark to display the number of posts published by the Author.', 'wp-author-status' ),
						'type'				=> 'checkbox',
						'default'			=> 'on',
						'sanitize_callback' => 'sanitize_text_field'
					),
					array(
						'name'				=> 'wpaust_howago',
						'label'				=> __( 'Display Time Ago:', 'wp-author-status' ),
						'desc'				=> __( 'Mark to display how long ago the author has updatd his status.', 'wp-author-status' ),
						'type'				=> 'checkbox',
						'default'			=> 'on',
						'sanitize_callback' => 'sanitize_text_field'
					),
				),
			);
			return $settings_fields;
		}

		/**
		 * Wraps the settings page for display.
		 *
		 * This method outputs the HTML for the settings page and sidebar content,
		 * providing a container for the settings fields and sidebar to be displayed.
		 *
		 * @return void
		 */
		public function wpaust_options_page() { ?>
			<div class="wpaust_settings_area">
				<div class="wrap wpaust_settings">
                    <h1 class="main-header"><?php esc_html_e( 'WP Author Status Options', 'wp-author-status' ); ?></h1>
                    <?php
					$this->settings_api->show_navigation();
					$this->settings_api->show_forms();
                    ?>
				</div>
				<?php $this->wpaust_admin_footer_note(); ?>
			</div>
			<?php
			$sidebar_content = $this->sidebar_info->wpaust_sidebar();
			echo wp_kses_post( $sidebar_content );
		}

		/**
		 * Includes the help page for the plugin.
		 *
		 * This method loads the HTML or template for the plugin's help page, providing 
		 * users with instructions or support information.
		 *
		 * @return void
		 */
		public function wpaust_help_page() {
            if ( !current_user_can( 'manage_options' ) )  {
                wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'wp-author-status' ) );
            }
            require_once ( WPAUST_PLUGIN_PATH . 'includes/wpaust-help.php' );

            $wpaust_help = new WPAUST_Help();
            $wpaust_help->render_wpaust_help_page();
		}

		/**
		 * Adds a custom footer note below the settings panel.
		 *
		 * This method appends a footer message to the settings page, typically used 
		 * for branding or providing additional information.
		 *
		 * @return void
		 */
		public function wpaust_admin_footer_note() {
			?>
			<div id="wpaust-narration" class="postbox-container">
				<div id="wpaustusage-note" class="wpaustusage-maincontent">
					<div class="wpaust"><?php
						printf(
							/* translators: %1$s: Opening HTML tag for heading, %2$s: Closing HTML tag for heading, %3$s: Opening paragraph tag, %4$s: Link to the Help page, %5$s: Contact Us link, %6$s: Closing paragraph tag, %7$s: Opening paragraph tag for feedback request, %8$s: Link to leave a review */
							esc_html__(
								'%1$sNeed Help with Setup?%2$s
								%3$sFor detailed instructions on setting up the Author Status plugin, please visit the %4$s. If you face any issues or have questions, feel free to %5$s and we\'ll assist you promptly.%6$s
								%7$sYour feedback matters! Please take a moment to rate your experience with the plugin. Your review helps us improve and provide better support. Thank you!%8$s',
								'wp-author-status'
							),
							'<h3>',
							'</h3>',
							'<p>',
							'<a href="' . esc_url(admin_url("admin.php?page=wpaust-help")) . '">Help page</a>',
							'<a href="' . esc_url("https://wordpress.org/support/plugin/wp-author-status/") . '" target="_blank">Contact Us</a>',
							'</p>',
							'<p class="likeit">',
							'<a target="_blank" href="' . esc_url("https://wordpress.org/support/plugin/wp-author-status/reviews/?filter=5/#new-post") . '">&#9733;&#9733;&#9733;&#9733;&#9733;</a></p>',
						);
						?>
					</div>
				</div>
			</div>
			<?php
		}

		/**
		 * Retrieves a list of pages from the WordPress site.
		 *
		 * This method fetches the pages available on the site and returns them as an 
		 * associative array with the page ID as the key and the page title as the value.
		 *
		 * @return array Array of page names with key-value pairs.
		 */
		public function get_pages() {
			$pages = get_pages();
			$pages_options = array();
			if ( $pages ) {
				foreach ( $pages as $page ) {
					$pages_options[$page->ID] = $page->post_title;
				}
			}
			return $pages_options;
		}
	}
endif;

$wpaust_settings_config = new WPAUST_Settings_Config();

/**
 * Retrieves the value of a settings field.
 *
 * This function fetches a specific option from the WordPress options table.
 * If the option is not found, it falls back to the specified default value.
 *
 * @param string $option The name of the settings field.
 * @param string $section The section name the field belongs to.
 * @param string $default The default value to return if the option is not found.
 *
 * @return mixed The value of the option or the default value.
 */
if ( !function_exists( 'wpaust_get_option' ) ) {
    function wpaust_get_option( $option, $section, $default = '' ) {

        $options = get_option( $section );

        if ( isset( $options[$option] ) ) {
            return $options[$option];
        }

        return $default;
    }
}